package cluster;

import index.Term;
import index.TermIDConflict;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import constants.Constant;

/**
 * This class contains information about clusters and basic manipulations. Each cluster may contain one or more
 * files and a set of terms.
 */
public class Cluster {
	/**
	 * A list of files which are in this cluster
	 */
	private List fileNames;
	
	/**
	 * The vector of terms(fingerprints) that represent the centroid of the cluster.
	 * This vector is not sparse i.e. we only hold the terms with weight <> 0 in cluster
	 */
	private List terms;

	public Cluster(String fileName, List fileTerms) {
		fileNames = new ArrayList();
		fileNames.add(fileName);
		terms = fileTerms;
	}
	
	/**
	 * Merging to clusters: merging list of files and update the centroid
	 * @param other The other cluster to be merged with this cluster
	 */
	public void merge(Cluster other) {
		try {
			for (Iterator it = terms.iterator(); it.hasNext();) {
				Term term = (Term) it.next();
				
				if(other.terms.contains(term)) {
					int otherTerm = other.terms.indexOf(term);
					
					term.merge((Term)other.terms.get(otherTerm), fileNames.size(), other.fileNames.size());
					
					other.terms.remove(otherTerm);
				} else {
					term.merge(null, fileNames.size(), other.fileNames.size());
				}
			}
			
			//Must not delete an item when iterate on it
			for (Iterator it = other.terms.iterator(); it.hasNext();) {
				Term term = (Term) it.next();
				term.merge(null, other.fileNames.size(), fileNames.size());
				terms.add(term);
			}
			
			other.terms.clear();
		} catch (TermIDConflict e) {
			System.out.println("The terms must have same ID!!");
			System.out.println(Constant.getString("Cluster.test"));
			e.printStackTrace(System.err);
			System.exit(1);
		}

		fileNames.addAll(other.fileNames);
	}
	
	public List getFileNames() {
		return fileNames;
	}

	public List getTerms() {
		return terms;
	}
}
